﻿using Holoville.HOTween;
using UnityEngine;

public class CameraFollow : MonoBehaviour {

	#region SINGLETON

	private static CameraFollow _instance;

	public static CameraFollow Instance {
		get { return _instance; }
	}

	public void Awake() {
		if (CameraFollow.Instance != null) {
			DestroyImmediate(gameObject);
			return;
		}
		_instance = this;
	}

	public void OnApplicationQuit() {
		_instance = null;
	}

	#endregion

	public Transform target;
	public Transform lookAtTarget;

	public bool doLookAt;
	public bool followTargetAngle;
	public bool rigidbodyMode;
	public float angle;

	public Vector3 lookOffset;
	public Vector3 positionOffset;
	public float height;
	public float distance;
	public float lookSmoothing;
	public float moveSmoothing;

	private Transform myTransform;

	private bool isOverridden;
	private Transform overrideTarget;
	private Transform overrideLookTarget;

	private float currentAngle;
	private float currentHeight;
	private Vector3 currentPosOffset;

	public const float DefaultDistance = 10;
	public const float DefaultHeight = 13;
	public const float DefaultLookSmoothing = 4;
	public const float DefaultMoveSmoothing = 4;

	private void Start() {
		myTransform = transform;
	}

	public void RevertToDefaults(float tweenTime = 0f) {
		if (tweenTime > 0f) {
			HOTween.Kill(this);
			HOTween.To(this, tweenTime,
					   new TweenParms().Prop("height", DefaultHeight).Prop("distance", DefaultDistance).Prop(
						"positionOffset", Vector3.zero).Prop("lookOffset", Vector3.zero).Prop("angle", 0f).Ease(
							EaseType.EaseInOutSine));
		} else {
			height = DefaultHeight;
			distance = DefaultDistance;
			positionOffset = Vector3.zero;
			lookOffset = Vector3.zero;
			angle = 0f;
		}
		followTargetAngle = false;
		lookSmoothing = DefaultLookSmoothing;
		moveSmoothing = DefaultMoveSmoothing;
	}

	public void OverrideCamera(Transform camPositionTarget, Transform camLookTarget, float moveSmoothing = DefaultMoveSmoothing) {
		isOverridden = true;
		this.moveSmoothing = moveSmoothing;
		overrideTarget = camPositionTarget;
		overrideLookTarget = camLookTarget;
	}

	private void LateUpdate() {
		if (isOverridden) {
			myTransform.position = Vector3.Lerp(myTransform.position, overrideTarget.position, moveSmoothing);
			myTransform.rotation = Quaternion.Lerp(myTransform.rotation, Quaternion.LookRotation((overrideLookTarget.position - myTransform.position).normalized), lookSmoothing);
		} else {

			if (followTargetAngle) {
				if (rigidbodyMode && target.rigidbody) {
					angle = 90 - Mathf.Atan2(target.rigidbody.velocity.z, target.rigidbody.velocity.x) * Mathf.Rad2Deg;
					//angle = Vector3.Angle(target.position, target.position + target.rigidbody.velocity.normalized);
				} else {
					angle = target.eulerAngles.y;
				}
			}
			currentAngle = Mathf.LerpAngle(currentAngle, angle, lookSmoothing * Time.deltaTime);
			currentHeight = Mathf.Lerp(currentHeight, height, moveSmoothing * Time.deltaTime);
			currentPosOffset = Vector3.Lerp(currentPosOffset, positionOffset, moveSmoothing * Time.deltaTime);
			myTransform.position = Vector3.Lerp(myTransform.position,target.position - (Quaternion.Euler(0, currentAngle, 0) * Vector3.forward * distance) +
									   Vector3.up * currentHeight + Quaternion.Euler(0, currentAngle, 0) * currentPosOffset, moveSmoothing * Time.deltaTime);
			if (doLookAt) {
				myTransform.LookAt((lookAtTarget != null) ? lookAtTarget.position : (target.position + Quaternion.Euler(0, angle, 0) * lookOffset));
			} else {
				myTransform.rotation =
					Quaternion.LookRotation(
						((target.position + Quaternion.Euler(0, angle, 0) * lookOffset) - myTransform.position).normalized);
			}
		}
	}

	public void StopOverride() {
		isOverridden = false;
	}

	public void SnapToTarget() {
		if (isOverridden) {
			myTransform.position = overrideTarget.position;
			myTransform.rotation =
				Quaternion.LookRotation((overrideLookTarget.position - myTransform.position).normalized);
		} else {
			currentAngle = angle;
			currentHeight = height;
		}
	}


}

